//	Random Name Generator
//	Written by:  Brian Lee Bowers
//	Based on pseudo code written by:  Stephen Zielinski

#include <time.h>
#include <stddef.h>
#include <stdlib.h>
#include <iostream.h>
#include <iomanip.h>
#include <strstream.h>

unsigned short icOcc[] = {	1, 7, 3, 3, 5, 8, 5, 3, 6, 7, 7,
				6, 2, 7, 7, 8, 5, 7, 1, 1, 1};

char icArr[] = {	'b', 'c', 'd', 'f', 'g', 'h', 'j',
			'k', 'l', 'm', 'n', 'p', 'q', 'r',
			's', 't', 'v', 'w', 'x', 'y', 'z'};

unsigned short vOcc[] = {	16, 25, 16, 18, 16, 9};

char vArr[] = {	'a', 'e', 'i', 'o', 'u', 'y'};

unsigned short fcOcc[] = {	1, 7, 3, 3, 5, 8, 5, 3, 6, 7, 7,
				6, 2, 7, 7, 8, 5, 7, 1, 1, 1};

char fcArr[] = {	'b', 'c', 'd', 'f', 'g', 'h', 'j',
			'k', 'l', 'm', 'n', 'p', 'q', 'r',
			's', 't', 'v', 'w', 'x', 'y', 'z'};

class RandGen
{
public:
  RandGen(unsigned long initSeed = TimeSeed());
  
  void SetSeed(unsigned long newSeed = TimeSeed());
  unsigned short operator () (unsigned short lim);
  
private:
  static unsigned long TimeSeed()
  {
    return (unsigned long)time(NULL);
  }
  
  unsigned long seed;
};

inline RandGen::RandGen(unsigned long initSeed)
{
  seed = initSeed;
}

inline void RandGen::SetSeed(unsigned long newSeed)
{
  seed = newSeed;
}

unsigned short RandGen::operator () (unsigned short lim)
{
  seed = seed * 5709421ul + 1ul;
  return (unsigned short)((seed >> 16ul) % lim);
}

RandGen rng;

//*********************************************************
//  Class:  Syllable
//  Purpose:  unit of a name
//  Written by:  Brian Lee Bowers
//  Revisions:  0.01  14 June 1998
//
//  0.01  a syllable can be any one of four different types:
//    CVC, CV, VC, or V.  A C represents a consonant, while
//    a V represents a vowel.  A syllable is constructed by
//    determining what type of syllable, initial consonant
//    (if any), vowel, and then final consonant (if any).
//
//*********************************************************

class Syllable
{
public:
  Syllable(void);             //  constructor
  ~Syllable(void);            //  destructor
  
  friend ostream& operator << (ostream& os, const Syllable& s);
  
private:
  void Display(ostream& os) const;
  
  char letts[4];
  unsigned int len;
};

//*********************************************************
//  Class:  Name
//  Purpose:  random name generation
//  Written by:  Brian Lee Bowers
//  Revisions:  0.01  14 June 1998
//
//  0.01  A name is a collection of syllables.  This class
//    will allow the user to determine how many syllables
//    a name will have.
//
//*********************************************************

class Name
{
public:
  Name(void);               //  default constructor
  Name(unsigned int);       //  constructor
  ~Name(void);              //  destructor
  
  friend ostream& operator << (ostream& os, const Name& n);
  
private:
  
  void Display(ostream& os) const;
  
  Syllable	*sylls;          //  dynamic array of syllables
  unsigned int len;         //  number of syllables
};

//*********************************************************
//  Class:  Syllable
//  Purpose:  unit of a name
//  Written by:  Brian Lee Bowers
//  Revisions:  0.01  14 June 1998
//
//  0.01  a syllable can be any one of four different types:
//    CVC, CV, VC, or V.  A C represents a consonant, while
//    a V represents a vowel.  A syllable is constructed by
//    determining what type of syllable, initial consonant
//    (if any), vowel, and then final consonant (if any).
//
//*********************************************************

Syllable::Syllable(void)
{
  const unsigned int syllT = 4;
  unsigned int i;
  short temp;
  unsigned short currLett = 0;
  unsigned short count;
  unsigned short sType = rng(100) + 1;
  static char sOcc[syllT] = {35, 30, 10, 25};
  for (i=0; i<4; i++)
    if (sType > sOcc[i])
      sType -= sOcc[i];
    else
      break;
  
  switch(i)
    {
    case 0 :	//  CVC syllable
      temp = rng(100) + 1;
      count = 0;
      while(1)
	{
	  temp -= icOcc[count];
	  if (temp <= 0)
	    {
	      letts[currLett] = icArr[count];
	      currLett++;
	      break;
	    };
	  
	  count++;
	};
    case 2 :	//  VC syllable
      temp = rng(100) + 1;
      count = 0;
      while(1)
	{
	  temp -= vOcc[count];
	  if (temp <= 0)
	    {
	      letts[currLett] = vArr[count];
	      currLett++;
	      break;
	    };
	  
	  count++;
	};
      
      temp = rng(100) + 1;
      count = 0;
      while(1)
	{
	  temp -= fcOcc[count];
	  if (temp <= 0)
	    {
	      letts[currLett] = fcArr[count];
	      currLett++;
	      break;
	    };
	  
	  count++;
	};
      break;
    case 1 :	//  CV syllable
      temp = rng(100) + 1;
      count = 0;
      while(1)
	{
	  temp -= icOcc[count];
	  if (temp <= 0)
	    {
	      letts[currLett] = icArr[count];
	      currLett++;
	      break;
	    };
	  
	  count++;
	};
      
      temp = rng(100) + 1;
      count = 0;
      while(1)
	{
	  temp -= vOcc[count];
	  if (temp <= 0)
	    {
	      letts[currLett] = vArr[count];
	      currLett++;
	      break;
	    };
	  
	  count++;
	};
      
      break;
    case 3 :	//  V syllable
      temp = rng(100) + 1;
      count = 0;
      while(1)
	{
	  temp -= vOcc[count];
	  if (temp <= 0)
	    {
	      letts[currLett] = vArr[count];
	      currLett++;
	      break;
	    };
	  
	  count++;
	};
      
      break;
    default :
      cerr << "Impossible syllable type.  Aborting.";
      exit(1);
    };
  
  letts[currLett] = '\0';
}

Syllable::~Syllable(void)
{}

ostream& operator << (ostream& os, const Syllable& s)
{
  s.Display(os);
  return os;
}

void Syllable::Display(ostream& os) const
{
  os << letts;
  return;
}

//*********************************************************
//  Class:  Name
//  Purpose:  random name generation
//  Written by:  Brian Lee Bowers
//  Revisions:  0.01  14 June 1998
//
//  0.01  A name is a collection of syllables.  This class
//    will allow the user to determine how many syllables
//    a name will have.
//
//*********************************************************

Name::Name(void)
{
  len = rng(6) + 1;
  sylls = new Syllable[len];
}

Name::Name(unsigned int ln)
{
  len = ln;
  sylls = new Syllable[len];
}

Name::~Name(void)
{
  delete [] sylls;
}

ostream& operator << (ostream& os, const Name& n)
{
  n.Display(os);
  return os;
}

void Name::Display(ostream& os) const
{
  for (unsigned int i=0; i<len; i++)
    os << sylls[i];
  return;
}

int main(void)
{
  Name names[26];
  char buffer[80];
  ostrstream ost(buffer, 80);
  for (unsigned int i=0; i<26; i++)
    {
      ost << names[i] << '\0';
      cout	<< setiosflags(ios::left)
		<<setw(25)
		<< buffer
		<< resetiosflags(ios::left);
      ost.seekp(ios::beg);
      if ((i%2) == 1)
	cout << endl;
    };
  return 0;
}
